editor.cfc
- recreates aspects of editor (insert,update,delete) and encapsulates them into a cfc
- allows override of schema settings by specifying a fieldlist, based on original sql.cfm code (not yet supported)
- these functions cfinclude files from _lib thereby retaining the option to copy and paste from _lib for agile custom dev
- this is for ms sql, additional cfcs could be made for mysql, etc.
	
	psReadRecord parameters
	TableName, RecordID, FieldList (optional, not yet supported - use to override default padsite schema)
	returns record(s) in the form of a query
	
	psInsertRecord parameters
	TableName, RecordID, FieldList (optional, not yet supported - use to override default padsite schema)
	- needs a pre-query loop, query loop and post-query loop from padsite
	returns Result (which includes new ID)
	
	psUpdateRecord parameters
	TableName, RecordID, FieldList (optional, not yet supported - use to override default padsite schema)
	- needs a pre-query loop, query loop and post-query loop from padsite
	returns Result (boolean)
	
	psDeleteRecord parameters
	TableName, RecordID, SoftDelete (optional, not yet supported, default = no, if yes, will create Deleted field if not present and then set to true)
	returns Result (boolean)
	
	psSearchRecords parameters
	TableName, Reporter
	generates search query, saves to database and forces window.opener to make http request for saved search
	

<cfcomponent>


<cfset PadsiteServices = createObject("component", "padsite.services.padsite")>


<cffunction name="psReadRecord">
	<cfargument name="TableName" type="string" required="true">
	<cfargument name="RecordID" type="numeric" required="true">
	<cfset local.Result.Success = false>
	<cfset local.Result.ErrorMessage = "">
	
	<cfquery name="local.Schema" datasource="#Application.psDataSource#">
  select * from #Application.psSchemaTable# where TableName = '#arguments.TableName#' order by Queue
  </cfquery>
  <cfquery name="local.Record" datasource="#Application.psDataSource#">
  select * from #arguments.TableName# where ID = #Val(arguments.ID)#
  </cfquery>
  
	<cfset local.Result.Success = true />
	<cfset local.Result.Record = local.Record>
	<cfset local.Result.RecordID = RecordID />
	<cfset local.Result.Schema = local.Schema />
	<cfreturn local.Result />
</cffunction>


<cffunction name="psInsertRecord" access="public" returnType="struct" hint="inserts a new record into TableName, returns RecordID and more">
	<cfargument name="TableName" type="string" required="true">
	<cfargument name="RecordID" type="numeric" required="false" default="0">
	<cfset var Results = StructNew() />
	<cfset var Insert = ""/>
	<cfset Result.Success = false />
	<cfset Result.RecordID = 0 />
	<cfset Result.Schema = StructNew() />
	<cfset Result.ErrorMessage = "" />
	
	<cfquery name="local.Schema" datasource="#Application.psDataSource#">
  select * from #Application.psSchemaTable# where TableName = '#arguments.TableName#' order by Queue
  </cfquery>
  <cftransaction>
  	<!--- perform pre-query loop -- set params --->
  	<cfinclude template="/padsite/services/_lib/_lib/editor_prequery_loop.cfm">
  	<!--- perform Update/Insert Queries (sets RecordID with updated record id) --->
  	<cfinclude template="/padsite/services/_lib/_lib/editor_query_loop.cfm">
  	<!--- perform post-query loop -- upload files, process nonstandard datatypes --->
  	<cfinclude template="/padsite/services/_lib/_lib/editor_postquery_loop.cfm">
  	<!--- parent tables are used by iframe summaries --->
  	<cfif IsDefined("form.parentid") and IsNumeric(form.parentid)>
  		<cfquery name="UpdateParentID" datasource="#Application.psDataSource#">
  		update #arguments.TableName# set #ParentTable#ID = #Val(form.parentid)# where ID = #arguments.RecordID#
  		</cfquery>
  	</cfif>
  </cftransaction>
  
	<cfset Results.Success = true />
	<cfset Results.RecordID = RecordID />
	<cfset Results.Schema = Schema />
	<cfreturn Results />
</cffunction>


<cffunction name="psUpdateRecord" access="public" returnType="struct" hint="updates a record in TableName, returns RecordID and more">
	<cfargument name="TableName" type="string" required="true">
	<cfargument name="RecordID" type="numeric" required="true">
	<cfset local.Result.Success = false>
	<cfset local.Result.ErrorMessage = "">
	
	<cfquery name="local.Schema" datasource="#Application.psDataSource#">
  select * from #Application.psSchemaTable# where TableName = '#arguments.TableName#' order by Queue
  </cfquery>
  <cfquery name="local.Table" dataSource="#Application.psDataSource#">
  select * from #arguments.TableName# where ID = #Val(arguments.RecordID)#
  </cfquery>
  
  <cftransaction>
		<!--- perform pre-query loop -- set params --->
		<cfinclude template="/padsite/services/_lib/_lib/editor_prequery_loop.cfm">
		<!--- perform Update/Insert Queries (sets RecordID with updated record id) --->
		<cfinclude template="/padsite/services/_lib/_lib/editor_query_loop.cfm">
		<!--- perform post-query loop -- upload files, process nonstandard datatypes --->
		<cfinclude template="/padsite/services/_lib/_lib/editor_postquery_loop.cfm">
		<!--- parent tables are used by iframe summaries --->
		<cfif IsDefined("form.parentid") and IsNumeric(form.parentid)>
			<cfquery name="UpdateParentID" datasource="#Application.psDataSource#">
			update #TableName# set #ParentTable#ID = #Val(form.parentid)# where ID = #RecordID#
			</cfquery>
		</cfif>
  </cftransaction>
	
	<cfset local.Result.Success = true>
	<cfset local.Result.RecordID = RecordID />
	<cfset local.Result.Schema = local.Schema />
	<cfreturn local.Result>
</cffunction>


<cffunction name="psDeleteRecord" access="public" returnType="struct" hint="deletes a record in TableName, returns RecordID and more">
	<cfargument name="TableName" type="string" required="true">
	<cfargument name="RecordID" type="numeric" required="true">
	<cfset local.Result.Success = false>
	<cfset local.Result.ErrorMessage = "">
	
	<cfquery name="local.Schema" datasource="#Application.psDataSource#">
  select * from #Application.psSchemaTable# where TableName = '#arguments.TableName#' order by Queue
  </cfquery>
  <cfquery name="local.Table" datasource="#Application.psDataSource#">
  select * from #arguments.TableName# where ID = #Val(arguments.RecordID)#
  </cfquery>
  <cfloop from="1" to="#ListLen(ValueList(local.Schema.FieldName))#" index="local.i">
  	<cfif local.Schema.DataType[local.i] eq "text" and Evaluate("local.Table."&local.Schema.FieldName[local.i]) neq "">
  		<cfif local.Schema.Subtype[local.i] eq "image" and FileExists(ExpandPath(ListGetAt(local.Schema.Options[local.i],1) & local.Table.ID & ".jpg"))>
  			<!--- needs to be updated to delete all sizes of image as well --->
  			<cffile action="delete" file="#ExpandPath(ListGetAt(local.Schema.Options[local.i],1) & local.Table.ID & ".jpg")#">
  			<cfelseif Schema.Subtype[local.i] eq "audio">
  			<cfelseif Schema.Subtype[local.i] eq "video">
  			<cfelseif Schema.Subtype[local.i] eq "pdf" and FileExists(ExpandPath(ListGetAt(local.Schema.Options[local.i],1) & local.Table.ID & ".pdf"))>
  			<cffile action="delete" file="#ExpandPath(ListGetAt(local.Schema.Options[local.i],1) & local.Table.ID & ".pdf")#">
  			<cfelseif Schema.Subtype[local.i] eq "file" and FileExists(ExpandPath(ListGetAt(local.Schema.Options[local.i],1) & local.Table.ID & "." & local.Schema.FieldName[local.i]))>
  			<cffile action="delete" file="#ExpandPath(ListGetAt(local.Schema.Options[local.i],1) & local.Table.ID & "." & local.Schema.FieldName[local.i])#">
  		</cfif>
  	</cfif>
  </cfloop>
  <cfquery name="local.Delete" datasource="#Application.psDataSource#">
  delete from #arguments.TableName# where ID = #Val(arguments.RecordID)#
  </cfquery>
  
	<cfset local.Result.Success = true>
	<cfset local.Result.RecordID = arguments.RecordID>
	<cfset local.Result.Schema = local.Schema>
	<cfreturn Result>
</cffunction>


<cffunction name="psSearchRecords" hint="performs search of TableName, saves as a recent search and directs window.opener to make http request for report of saved search">
	<cfargument name="TableName" type="string" required="true">
	<cfargument name="Reporter" type="string" required="true">
	<!--- <cfset var TableName = arguments.TableName>
	<cfset var Reporter = arguments.Reporter> --->
	<cfset local.SearchType = "">
	<cfset local.QueryString = "">
	<cfset local.SearchName = "">
	<cfset local.InsertSearch = "">
	<cfset local.SearchMode = true>
	<cfset local.Result.Success = false>
	<cfset local.Result.ErrorMessage = "">
	
	<cfquery name="local.Schema" datasource="#Application.psDataSource#">
  select * from #Application.psSchemaTable# where TableName = '#TableName#' order by Queue
  </cfquery>
  <!--- <cfquery name="local.Table" datasource="#Application.psDataSource#">
  select * from #arguments.TableName# where ID = #Val(arguments.RecordID)#
  </cfquery> --->
	<!--- perform pre-query loop -- set params --->
	<cfinclude template="/padsite/services/_lib/_lib/editor_prequery_loop.cfm">
	<!--- make and save search then send http request via window.opener for saved search --->
	<cfinclude template="/padsite/services/_lib/_lib/editor_generate_search.cfm">
	
	<cfset local.Result.Success = true>
	<cfset local.Result.SearchName = local.SearchName>
	<cfset local.Result.SearchID = local.InsertSearch.RecordID>
	<cfreturn Result>
</cffunction>


</cfcomponent>