reporter.cfc
- recreate aspects of reporter (psRead) and encapsulate them into a cfc
- these functions should use cf_include and _lib thereby retaining the option to copy and paste for agile custom dev
- this is for ms sql, additional cfcs could be made for mysql, etc.

	psReadRecords parameters
	TableName {, Sort, Filter, FieldList (use to override schema)}
	returns record(s) in the form of a query
	- psQueryTable is called from psSearchTable when using a saved search? always?
	
	psBackup parameters
	TableName, Format requested?
	returns Result (sql code for recreating table and data)
	
	psExport parameters
	TableName, Format requested (xml,csv,xls)
	returns Result (text or xml object?)


<cfcomponent>


<cfset PadsiteServices = createObject("component", "padsite.services.padsite")>


<cffunction name="psReadRecords" access="public" returnType="struct" hint="queries database for users">
	<cfargument name="TableName" type="string" required="true">
	<cfset local.Result.Query = "">
	<cfset local.Result.TotalRecords = 0>
	<cfset local.Result.HasVisibleQueueField = false>
	<cfset local.Result.FilterMenu = "">
	<cfset local.Result.Schema = "">
	<cfset local.Result.DisplaySendMail = false>
	<!--- <cfset local.DisplayParentSendmail = false> --->
	<cfset local.Result.DisplayParentSendmail = local.DisplayParentSendmail = false>
	<cfset local.Result.Success = false>
	<cfset local.Result.ErrorMessage = "">
	
	<!--- get a query of TableName's schema --->
	<cfset local.Result.Schema = PadsiteServices.psGetSchema(arguments.TableName)>
	
	<!--- check schema for specific field types - queue, emails, foreign keys, check for filter --->
	<cfset local.Result.HasVisibleQueueField = PadsiteServices.psHasVisibleQueueField(arguments.TableName)>
	<cfset local.Result.DisplaySendmail = PadsiteServices.psHasEmailAddressField(arguments.TableName)>
	<!--- <cfset local.Result.ForeignKeys = PadsiteServices.psForeignKeys(arguments.TableName)> --->
	<cfset local.Result.FilterMenu = PadsiteServices.psGetFilterMenu(arguments.TableName)>
	<!--- <cfset local.Result.DisplayParentSendmail = local.DisplayParentSendmail> --->
	
	<!--- perform select query as requested by user --->
	<cfparam name="url.select" default="-1">
	<cfparam name="url.order" default="asc">
	<cfif local.Result.HasVisibleQueueField>
	  <cfparam name="url.sort" default="Queue">
	  <cfelse>
	  <cfparam name="url.sort" default="ID">
	</cfif>

	<!--- when searching groups, list of MemberIDs is hard-coded into search... however, because there is only one query string being saved
	and because fields of type text cannot be used as ID lists in queries, it is necessary to either put in a placeholder that tells
	the search to split it into two queries (which would be bad if there was more than one type of grouping), the optimal solution
	would be to create an intermediate table that will relate records in the searched table to records of groups in another table
	ID				MemberID	GroupID
	1					3					2
	2					5					2
	3					5					4
	to start doing grouplists in this way, would require a more complex GroupsTable setup (requiring always 2 tables instead of 1)
	the upside is that I would be able to say 'and ID in (select MemberID from #GroupsTable# where GroupID = #GroupID#)' --->
	
	<cfif IsDefined("url.select") and Find("search",url.select)>
	  <!--- perform search query --->
	  <cfquery name="local.Search" datasource="#Application.psDataSource#">
	  select * from #Application.psSearchTable# where ID = #Val(ListGetAt(url.select,2,"."))#
	  </cfquery>
	  <cfquery name="local.Table" datasource="#Application.psDataSource#">
	  #PreserveSingleQuotes(local.Search.QueryString)# order by #SpanExcluding(url.sort,Application.psIllegal_URL_Chars)# #SpanExcluding(url.order,Application.psIllegal_URL_Chars)#
	  </cfquery>
	  <cfelseif IsDefined("url.select") and (url.select gte 0 or url.select eq "")><!---  --->
	  <!--- perform filtered query --->
	  <cfif local.Result.FilterMenu.Query.Subtype eq "filter_foreignkey">
	  	<cfquery name="local.Table" datasource="#Application.psDataSource#">
	  	select * from #arguments.TableName# where #local.Result.FilterMenu.FilterForeignKey# = #Val(url.select)# order by #SpanExcluding(url.sort,Application.psIllegal_URL_Chars)#
	  	</cfquery>
	  	<!--- <cfelseif local.Result.FilterMenu.Query.Subtype eq "filter_group">
	  	<cfquery name="local.FilterList" datasource="#Application.psDataSource#">
	  	select #local.FilterMenu.FilterIDList# as IDList from #local.FilterMenu.FilterTableName# where ID = #Val(url.select)#
	  	</cfquery>
	  	<cfquery name="local.Table" datasource="#Application.psDataSource#">
	  	select * from #arguments.TableName# where ID in (#IIf(ListLen(local.FilterList.IDList) gte 1,DE(local.FilterList.IDList),DE("0"))#) order by #SpanExcluding(url.sort,Application.psIllegal_URL_Chars)#
	  	</cfquery> --->
	  	<cfelseif local.Result.FilterMenu.Query.Subtype eq "filter_string">
	  	<cfquery name="local.Table" datasource="#Application.psDataSource#">
	  	select * from #arguments.TableName# where #local.Result.FilterMenu.FilterFieldName# = '#SpanExcluding(url.select,Application.psIllegal_URL_Chars)#' order by #SpanExcluding(url.sort,Application.psIllegal_URL_Chars)#
	  	</cfquery>
	  	<cfelse>
	  	<cfabort showerror="Unsupported Filter Subtype: Cannot Form Query">
	  </cfif>
	  <cfelse>
	  <!--- perform default query --->
	  <cfquery name="local.Table" datasource="#Application.psDataSource#">
	  select * from #arguments.TableName#
	  <cfif IsDefined("url.parent") and IsNumeric(url.parent) and IsDefined("ParentTable")>
	  	<cfset local.Result.HasParentTable = true>
	  	where #ParentTable#ID = #Val(url.parent)#
	  	<cfelse>
	  	<cfset local.Result.HasParentTable = false>
	  </cfif>
	  order by #SpanExcluding(url.sort,Application.psIllegal_URL_Chars)# #SpanExcluding(url.order,Application.psIllegal_URL_Chars)#
	  </cfquery>
	</cfif>
	<cfquery name="local.Total" datasource="#Application.psDataSource#">
	select ID from #arguments.TableName#
	</cfquery>
	<cfset local.Result.Query = local.Table>
	<cfset local.Result.TotalRecords = local.Total.RecordCount>
	<cfset local.Result.Success = true>
	<cfreturn local.Result>
</cffunction>


<cffunction name="psSendMail" returnType="struct">
	<!--- psSendMail gets most of its info from the form scope --->
	<cfset local.Result.Success = false>
	<cfset local.Result.ErrorMessage = "">
	<cfset local.Result.ValidAddresses = "">
	<cfset local.Result.InvalidAddresses = "">

	<cfparam name="form.recipients_selected" default="0">
  <cfparam name="form.recipients_other" default="0">
  <cfif form.recipients_selected eq 1>
  	<cfquery name="local.EmailAddress" dataSource="#Application.psDataSource#">
  	select top 1 FieldName from #Application.psSchemaTable# where TableName = '#SpanExcluding(url.source,Application.psIllegal_URL_Chars)#' and Subtype = 'email'
  	</cfquery>
  	<cfquery name="local.Recipients" dataSource="#Application.psDataSource#">
  	select #local.EmailAddress.FieldName# from #SpanExcluding(url.source,Application.psIllegal_URL_Chars)# where ID in (#form.recipient_ids#)
  	</cfquery>
  	<cfset local.RecipientList = Evaluate("ValueList(local.Recipients.#local.EmailAddress.FieldName#)")>
  	<cfelse>
  	<cfset local.RecipientList = "">
  </cfif>
  <cfif form.recipients_other eq 1>
  	<cfset local.RecipientList = ListAppend(local.RecipientList,form.recipients_other_addresses)>
  </cfif>
  <!--- insert data collected so far and retrieve ID number for file handling and finally update record when finished --->
  <cfquery name="local.Insert" dataSource="#Application.psDataSource#">
  insert into #Application.psMessageLogTable# (Sender,SenderAsEntered,Selector,RecipientList,Subject,Message,Sent,Importance) values ('#Session.Username#','#form.message_sender_address#','#form.sendmail#','#local.RecipientList#','#form.subject#','#form.message#',#Now()#,'#form.message_importance#');
  select max(ID) as NewID from #Application.psMessageLogTable#
  </cfquery>
  <!--- upload attached files --->
  <cfif form.attachedFile neq "">
  	<cfset local.AttachmentsDirectoryForThisMessage = ExpandPath("#Application.psEmailAttachmentsDirectory#" & "#local.Insert.NewID#/")>
  	<cfif not DirectoryExists(local.AttachmentsDirectoryForThisMessage)>
  		<cfdirectory action="create" directory="#local.AttachmentsDirectoryForThisMessage#">
  	</cfif>
  </cfif>
  <cfif form.attachedFile neq "">
  	<!--- upload to EmailAttachmentsDirectory (defined in application.cfm) --->
  	<cffile action="upload" fileField="form.attachedFile" destination="#local.AttachmentsDirectoryForThisMessage & "temp.tmp"#">
  	<cfset local.RevisedFileNameForThisAttachment = Replace(XMLFormat(cffile.clientFile)," ","_","all")>
  	<cffile action="rename" source="#local.AttachmentsDirectoryForThisMessage & "temp.tmp"#" destination="#local.RevisedFileNameForThisAttachment#">
  	<cfset local.AttachedFile = local.RevisedFileNameForThisAttachment>
  	<cfelse>
  	<cfset local.AttachedFile = "">
  </cfif>
  <cfloop list="#local.RecipientList#" index="local.CurrentAddress">
  	<cfif REFindNoCase("^[A-Z0-9][\w\.-]*@[A-Z0-9][\w\.-]*[A-Z0-9]\.[A-Z][A-Z\.]*[A-Z]$",Trim(local.CurrentAddress),1) gt 0>
  		<!--- split into two cfmailparts? text and html --->
  		<cfmail to="#local.CurrentAddress#" from="#form.message_sender_address#" subject="#form.subject#" server="#Application.psMailServer#" type="html">
  			<style>
  				body {
  					padding:40px;
  					font-family: Verdana;
  					font-size: 12px;}
  			</style>
  			#Replace(form.message,chr(13),"<br>","all")#
  			<cfmailparam name="Importance" value="#form.message_importance#">
  			<!--- return receipt: <cfmailparam name="Disposition-Notification-To" value='"Name" <#Session.Username#>'> --->
  			<cfif local.AttachedFile neq "">
  				<cfmailparam file="#Application.psEmailAttachmentsDirectory##local.Insert.NewID#/#local.AttachedFile#">
  			</cfif>
  		</cfmail>
  		<cfset local.Result.ValidAddresses = ListAppend(local.Result.ValidAddresses,local.CurrentAddress)>
  		<cfelse>
  		<cfset local.Result.InvalidAddresses = ListAppend(local.Result.InvalidAddresses,local.CurrentAddress)>
  	</cfif>
  </cfloop>
  <!--- update MessageLogTable with file list and remaining details --->
  <cfquery name="local.UpdateMessageLog" dataSource="#Application.psDataSource#">
  update #Application.psMessageLogTable# set RecipientResults = 'ValidAddresses:#local.Result.ValidAddresses#|InvalidAddresses:#local.Result.InvalidAddresses#', Attachment = '#local.AttachedFile#' where ID = #local.Insert.NewID#
  </cfquery>
	
		<cfset local.Result.Success = true>
		<cfreturn local.Result>
</cffunction>


</cfcomponent>
